/*============================================================================
/ psionfs.c
/ © Michael Pieper 30. Oct. 1999
/     Last change: 12. Nov. 2000 Michael Pieper
/============================================================================
/
/	This c-File includes all central Filesystem-Functions for the PSION-FileSystem
/	I started with this file, and have to split this file into good parts.
/
/============================================================================*/
#define DEBUG_FILE "/var/log/rfsv-client.log"

typedef long long dr9_off_t;
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
#include <KernelExport.h>
#include <time.h>
#include <malloc.h>
#include "fsproto.h"
#include "lock.h"
#include "cache.h"

#include <fs_info.h>		/* Diese Datei enthält die Strukturdefinition von fs_info (Filesysteminfo)	*/
#include <TypeConstants.h>	/* We need this for all MIME-Type Definitions								*/
#include <GraphicsDefs.h>	/* To inculde the Bitmaps we have to include this Headerfile				*/
#include "../Icons/PsionBitmaps.h"	/* The Icondefinitions												*/

#include "psionfs_attr.h"	/* Including all the Attribute-Filesystem-Functions							*/
#include "psionfs_dir.h"	/* Including all the Directory-Filesystem-Functions							*/
#include "psionfs_file.h"	/* Including all the File-Filesystem-Functions								*/
#include "psionfs_rfsv.h"	/* Including all PSION-Commands												*/

/*zu debugzwecken!*/
#include "../../DebugWin/debug.h"
#ifdef DEBUG_FILE
void dbg_print(char *head, size_t len, unsigned char *buffer);
#endif
/*=================================*/

#include "psionfs.h"
#include "../messages.h"	/* Die definierten Nachrichtentypen zwischen NCP und Client	*/
#include "mc_rfile.h"		/* Das RFSV-Protokoll	*/
#include "p_file.h"			/* dito.	*/

/*=================================*/


void setup_vnode(vnode *node, unsigned char *finfo);


/*  Start of fundamental (read-only) required functions */
static int		fs_mount(nspace_id nsid, const char *device, ulong flags,
						void *parms, size_t len, void **data, vnode_id *vnid);
static int		fs_unmount(void *_ns);

static int		fs_walk(void *_ns, void *_base, const char *file,
						char **newpath, vnode_id *vnid);

static int		fs_read_vnode(void *_ns, vnode_id vnid, char r, void **node);
static int		fs_write_vnode(void *_ns, void *_node, char r);
static int		fs_open(void *_ns, void *_node, int omode, void **cookie);

/* fs_access - checks permissions for access.	*/
static int		fs_access(void *_ns, void *_node, int mode);

/* fs_rfsstat - Fill in fs_info struct for device.	*/
static int		fs_rfsstat(void *_ns, struct fs_info *);

/* fs_readlink - Read in the name of a symbolic link.	*/
static int 		fs_readlink(void *_ns, void *_node, char *buf, size_t *bufsize);
/* End of fundamental (read-only) required functions. */
static int		fs_remove_vnode(void *ns, void *node, char r);
static int		fs_secure_vnode(void *ns, void *node);

static int		fs_rename(void *ns, void *olddir, const char *oldname,
					void *newdir, const char *newname);

#if 0
static int		fs_ioctl(void *ns, void *node, void *cookie, int cmd,
						void *buf, size_t len);
static int		fs_wfsstat(void *ns, struct fs_info *);
static int		fs_sync(void *ns);
static int     	fs_initialize(const char *devname, void *parms, size_t len);
#endif 



/* vnode_ops struct. Fill this in to tell the kernel how to call
	functions in your driver.
*/

vnode_ops fs_entry =  {
	&fs_read_vnode,						/* read_vnode func ptr	*/
	&fs_write_vnode,					/* write_vnode func ptr	*/
	&fs_remove_vnode,					/* remove_vnode func ptr	*/
	&fs_secure_vnode,					/* secure_vnode func ptr	*/
	&fs_walk,							/* walk func ptr	*/
	&fs_access,							/* access func ptr	*/
	&psionfs_create,					/* create func ptr	*/
	&psionfs_mkdir,						/* mkdir func ptr	*/
	NULL,								/* symlinc func ptr	*/
	NULL,								/* link func ptr	*/
	&fs_rename,							/* rename func ptr	*/
	&psionfs_unlink,					/* unlink func ptr	*/
	&psionfs_rmdir, 					/* rmdir func ptr	*/
	&fs_readlink,						/* readlink func ptr	*/
	&psionfs_opendir,					/* opendir func ptr	*/
	&psionfs_closedir,					/* closedir func ptr	*/
	&psionfs_free_dircookie,			/* free_dircookie func ptr	*/
	&psionfs_rewinddir,					/* rewinddir func ptr	*/
	&psionfs_readdir,					/* readdir func ptr	*/
	&fs_open,							/* open file func ptr	*/
	&psionfs_close,						/* close file func ptr	*/
	&psionfs_free_cookie,				/* free cookie func ptr	*/
	&psionfs_read,						/* read file func ptr	*/
	&psionfs_write,						/* write file func ptr	*/
	NULL, 								/* readv */
	NULL, 								/* writev */
	NULL,								/* ioctl func ptr	*/
	NULL,								/* setflags file func ptr	*/
	&psionfs_rstat,						/* rstat func ptr	*/
	&psionfs_wstat, 					/* wstat func ptr	*/
	NULL,								/* fsync	*/
	NULL, 								/* initialize func ptr	*/
	&fs_mount,							/* mount func ptr	*/
	&fs_unmount,						/* unmount func ptr	*/
	NULL,								/* sync func ptr	*/
	&fs_rfsstat,						/* rfsstat func ptr	*/
	NULL,								/* wfsstat func ptr	*/
	NULL,								/* select	*/
	NULL,								/* deselect	*/
	NULL,								/* open_indexdir	*/
	NULL,								/* close_indexdir	*/
	NULL,								/* free_indexdircookie	*/
	NULL,								/* rewind_indexdir	*/
	NULL,								/* read_indexdir	*/
	NULL,								/* create_index	*/
	NULL,								/* remove_index	*/
	NULL,								/* rename_index	*/
	NULL,								/* stat_index	*/

	/* All the Attribute Functions are defined int the psionfs_attr.c - File */
	&psionfs_open_attrdir,				/* open_attrdir	*/
	&psionfs_close_attrdir,				/* close_attrdir	*/
	&psionfs_free_attrdircookie,		/* free_attrdircookie	*/
	&psionfs_rewind_attrdir,			/* rewind_attrdir	*/
	&psionfs_read_attrdir,				/* read_attrdir	*/
	&psionfs_write_attr,				/* write_attr	*/
	&psionfs_read_attr,					/* read_attr	*/
	&psionfs_remove_attr,				/* remove_attr	*/
	&psionfs_rename_attr,				/* rename_attr	*/
	&psionfs_stat_attr,					/* stat_attr	*/

	/* No query functions available at the moment in this filesystem	*/
	NULL,								/* open_query	*/
	NULL,								/* close_query	*/
	NULL,								/* free_querycookie	*/
	NULL								/* read_query	*/
};

/*=======================================================================================
/	We need some informations in global variables. These variables are defined here!
=======================================================================================*/

int32			api_version = B_CUR_FS_API_VERSION;
static char		*gFSName = "psionfs";					/* My FilesystemName	*/
thread_id		gSerializeThread;						/* ?? 					*/
static char		debugFile[128];							/* in this file we have to write the debuginfo */

static int 
fs_mount(nspace_id nsid, const char *device, ulong flags, void *parms,
		size_t len, void **data, vnode_id *vnid)
{
	/*
	Kernel passes in nspace_id, (representing a disk or partition?)
	and a string representing the device (eg, "/dev/scsi/disk/030/raw)
	Flags will be used for things like specifying read-only mounting.
	parms is parameters passed in as switches from the mount command, 
	and len is the length of the otions. data is a pointer to a 
	driver-specific struct that should be allocated in this routine. 
	It will then be passed back in by the kernel to a number of the other 
	fs driver functions. vnid should also be passed back to the kernel, 
	representing the vnode id of the root vnode.
*/ 
	vnode *vnod;
	char text[128];
	int	result = EINVAL;
	nspace *nsp;
	size_t	l;
	MNT_INFO *mnt_info = (MNT_INFO *)parms;	/* Es wird der Port übergeben unter dem der NCP erreichbar ist.;	*/
	size_t largeIconLen = sizeof(kLargePsionIconBits);
	size_t smallIconLen  = sizeof(kSmallPsionIconBits);
	
#ifdef DEBUG_FILE
	int fd_debug;
	acquire_sem(mnt_info->sem);
	if ((fd_debug = open(DEBUG_FILE, O_RDWR|O_CREAT, 7770)) > 0) {
		write(fd_debug, "Mount gestartet!\n", 17);
		close(fd_debug);
	}	
	release_sem(mnt_info->sem);
#endif

	sprintf(text, "## fs_mount: dev=>%s< nsid=%ld\n", device, nsid);
	debug_print(text);

	if (mnt_info->pid_read < B_NO_ERROR || mnt_info->pid_write < B_NO_ERROR) {
		debug_print("-- fs_mount: Falsche PortID's übergeben!\n");
		return result;
	}
	
	nsp = (nspace *) malloc(sizeof(nspace));
	if (nsp == NULL) return ENOMEM;
	
	nsp->id = nsid;
	nsp->pid_read = mnt_info->pid_read;
	nsp->pid_write = mnt_info->pid_write;
	nsp->cid = mnt_info->cid;
	strcpy(nsp->device, mnt_info->device); 
	nsp->port_sem = mnt_info->sem;	/* This semaphore is set from the mounter. it is unique for the serial port!	*/
									/* gIf the mounter dies, it can be recognized through this semaphore!	*/
	nsp->dev_sem = create_sem(1, "PSION Device");
	nsp->fsi = NULL;				/* filesysteminfo not available at the moment	*/
	nsp->pb_running = true;			/* PalmBeach is up and running	*/
	
	sprintf(text, "-- fs_mount: semaphoreId: %ld\n", nsp->dev_sem);
	debug_print(text);

	if (mnt_info->debugFile != NULL) strcpy(debugFile, mnt_info->debugFile);	/* We have to remember the debugFilename!	*/
	else *debugFile = '\0';
	
	*data = nsp;

	vnod = search_vnode(nsp, vnid, 0, "\\", NULL);	/* Der Root-Node wird gesetzt!	*/
	nsp->rootDir = vnod;				/* Den Root-Node merken!	*/
	
	psionfs_write_attr(nsp, nsp->rootDir, LARGE_ICON_NAME, LARGE_ICON_TYPE, kLargePsionIconBits, &largeIconLen, (off_t)0);
	psionfs_write_attr(nsp, nsp->rootDir, SMALL_ICON_NAME, SMALL_ICON_TYPE, kSmallPsionIconBits, &smallIconLen, (off_t)0);

	l = sizeof(PSION_VOLUME);
	psionfs_write_attr(nsp, nsp->rootDir, BEOS_MIME_NAME, BEOS_MIME_TYPE, PSION_VOLUME, &l, (off_t)0);

	/* You MUST do this. Create the vnode for the root.	*/
	/* Dieser Funktionsaufruf stellt einen Speicherbereich für den Vnode zur Verfügung!	*/

	result = new_vnode(nsid, *vnid, (void*)vnod);
	if (result != B_NO_ERROR)
	{
		free(vnod);
		result = EINVAL;
	}
	else 
	{
		result = B_NO_ERROR;	
	}		

	sprintf(text, "-- fs_mount: ended: *vnid=%Ld\n", *vnid);
	debug_print(text);

	return result;
}

static int		
fs_unmount(void *_ns)
{
	nspace* ns = (nspace*)_ns;
	int result = B_NO_ERROR;
	
	debug_print("## fs_unmount\n");
	
	delete_sem(ns->dev_sem);	/* Delete the Device_Semaphore	*/
	if (ns->fsi != NULL) free (ns->fsi);
	free (ns);					/* free all the needed Space	*/
	
	return result;
}

/* fs_rfsstat - Fill in fs_info struct for device.	*/
static int		
fs_rfsstat(void *_ns, struct fs_info *fss)
{
	nspace* ns = (nspace*)_ns;
	unsigned char buffer[10], *inc_buf;
	int				result = ENOMEM;
	size_t			len;
	status_t		port_error;
	char text[128];
	
	sprintf(text, "## fs_rfsstat: %s\n", ns->device);
	debug_print(text);
	acquire_sem(ns->dev_sem);	/* be shure, noone else wants to read the structure! */
	
	if (ns->fsi == NULL) {							/* The filesysteminfo isn't available	*/
		ns->fsi = (fs_info *) malloc (sizeof(fs_info));
		
		if (ns->fsi == NULL) return ENOMEM;			/* not enough memory	*/
		
		sprintf(text, "-- fs_rfsstat: Calling PSION!\n");
		debug_print(text);

		WordToBytes(RF_STATUSDEVICE, buffer);			/* Der Befehl RF_STATUSDEVICE	*/
		WordToBytes(strlen(ns->device)+1, &buffer[2]);	/* Länge abhängig vom Devicenamen + Nullbyte	*/
		strcpy((char *)&buffer[4], ns->device);			/* Infos zum internen Laufwerk	*/

		inc_buf = send_command(ns, buffer, &len, &port_error);
		if (inc_buf != NULL && len >= 20) {
			if (BytesToWord(&inc_buf[4]) == 0) result = 0;
	
			/* Fill in device id.	*/
			ns->fsi->dev = ns->id;
		
			/* Root vnode ID	*/
			ns->fsi->root = ns->rootDir->id;
	
			/* File system flags.	*/
			ns->fsi->flags =	B_FS_IS_PERSISTENT | B_FS_IS_REMOVABLE |
								B_FS_HAS_MIME | B_FS_HAS_ATTR;

			switch (BytesToUword(&inc_buf[8]) & 0x00FF) {
				case P_FMEDIA_ROM:
				case P_FMEDIA_WRITEPROTECTED:
					ns->fsi->flags |= B_FS_IS_READONLY;
					debug_print("-- Filesystem is Readonly!\n");
					break;
			}

			/* FS block size.	*/
			ns->fsi->block_size = 1;
	
			/* IO size - specifies buffer size for file copying	*/
			ns->fsi->io_size = MAX_BLOCK;
	
			/* Total blocks?	*/
			ns->fsi->total_blocks = BytesToUlong(&inc_buf[12]);
	
			/* Free blocks = 0, read only	*/
			ns->fsi->free_blocks = BytesToUlong(&inc_buf[16]);
	
			/* Device name.	*/
	
			strncpy(ns->fsi->device_name, ns->device, sizeof(ns->fsi->device_name)-1);
			strncpy(ns->fsi->volume_name, (char *)&inc_buf[20], sizeof(ns->fsi->volume_name)-1);

			/* File system name	*/
			strcpy(ns->fsi->fsh_name,"psionfs");

			free(inc_buf);
		}
	}
	memcpy(fss, ns->fsi, sizeof(fs_info));	/* copy the values in the structure	*/
	release_sem(ns->dev_sem);
	
	sprintf(text, "-- fs_rfsstat: %s ended\n", ns->fsi->volume_name);
	debug_print(text);
	return 0;
}

/* fs_walk - the walk function just "walks" through a directory looking for
	the specified file. When you find it, call get_vnode on its vnid to init
	it for the kernel.
*/

static int		
fs_walk(void *_ns, void *_base, const char *file, char **newpath, vnode_id *vnid)
{
	char text[128];
	nspace*		ns = (nspace*)_ns;
	int			result = ENOENT;
	vnode*		base_vnod = (vnode *)_base;
	vnode*		newNode = NULL;

	/* Starting at the base, find file in the subdir, and return path
		string and vnode id of file. */
	sprintf(text, "## fs_walk: file:>%s< vnodbase:%Ld\n", file, base_vnod->id);
	debug_print(text);

	if (!ns->pb_running) {
		sprintf(text, "-- fs_walk: Port-PANIC, Result:%x\n", result);
		debug_print(text);

/*		return B_IO_ERROR;*/
	}

/*	if (strcmp(file,".") == 0) 					*vnid = base_vnod->id;
	else if (strcmp(file, "..") == 0) 			*vnid = base_vnod->parID;
	else {*/
		if (search_vnode(ns, vnid, base_vnod->id, file, NULL) == NULL) {
/*		newNode = search_vnode(ns, vnid, base_vnod->id, file, NULL);	*//* der node wird gesetzt	*/
/*		if (newNode == NULL) {*/
			sprintf(text, "-- fs_walk: vnid:*not found*, Result:%x\n", result);
			debug_print(text);

			return result;											/* Datei nicht vorhanden!	*/
		}
/*	}*/

	if (get_vnode(ns->id, *vnid, (void**)&newNode) != 0) result = EINVAL;
	else result = B_NO_ERROR;		

	if (newNode != NULL) {
		sprintf(text, "-- fs_walk: vnid:%Ld, size:%ld, result:%d\n", newNode->id, newNode->size, result);
	} else { 
		sprintf(text, "--fs_walk: result:%d\n", result);
	}
	debug_print(text);

	return result;
}

/* fs_read_vnode - Using vnode id, read in vnode information into fs-specific struct,
					and return it in node. the reenter flag tells you if this function
					is being called via some other fs routine, so that things like 
					double-locking can be avoided.	*/
static int		
fs_read_vnode(void *_ns, vnode_id vnid, char reenter, void **node)
{
	char text[128];
	int			result = B_NO_ERROR;
	vnode* 		newNode = (vnode *)vnid;	/* vnid = Adresse des node!	*/

	sprintf(text, "## fs_read_vnode vnode_id:%Ld, reenter:%d\n", vnid, reenter);
	debug_print(text);

	if (newNode != NULL) {
		newNode->read_vnode = true;			/* Durch read_vnode angefordert!	*/
		*node = (void *)newNode;
		if (newNode->deleted) result = B_ENTRY_NOT_FOUND;
	} else result = ENOMEM;
	
	
	return result;
}

/* gibt den node wieder frei!	*/
static int		
fs_write_vnode(void *_ns, void *_node, char reenter)
{
	char	text[128];
	nspace* ns = (nspace*)_ns;
	vnode	*node = (vnode*)_node;
	char 	*c;
	psion_attr	*at;
	
	sprintf(text, "## fs_write_vnode vnode_id:%Ld, reenter:%d\n", node->id, reenter);
	debug_print(text);
	
	if (node != NULL) {  
		/* hier muß noch einiges getan werden, wenn die VNODES wieder freigegeben werden sollen.	*/
		/* vor allem dann, wenn ein ROOT freigegeben wird, und die Childs noch nicht....	*/
		if (node->child != NULL) {
			sprintf(text, "!! fs_write_vnode: Problem! Want to release node with child! NodeId:%Ld, ChildId:%Ld\n", node->id, node->child);
			debug_print(text);
		}
		if (node->parID != 0) {
			if (node->prev->child == node) node->prev->child = node->next;	/* the node was a child	*/
			else node->prev->next = node->next;

			if (node->next != NULL) node->next->prev = node->prev;		/* der Letzte der Kette!	*/
			
		}
		if ((c = node->filename) != NULL) {
			node->filename = NULL;
			free(c);
		}
		if ((at = node->attr) != NULL) {
			node->attr = NULL;
			free_all_attr(at);
		}
		free(node);
	}
	return B_NO_ERROR;
}

static int		
fs_remove_vnode(void *_ns, void *_node, char r)
{
	char text[128];
	char		file[128], *c;
	int			result = B_NO_ERROR;
	vnode* 		node = (vnode *)_node;	
	nspace* 	ns = (nspace*)_ns;
	psion_attr	*at;

	sprintf(text, "## fs_remove_vnode vnode_id:%Ld, r:%x\n", node->id, r);
	debug_print(text);

	find_path(ns, node, file, sizeof(file));	/* find the complete filename	*/
	psion_delete (ns, file);
	
	if (node != NULL) {  
		/* hier muß noch einiges getan werden, wenn die VNODES wieder freigegeben werden sollen.	*/
		/* vor allem dann, wenn ein ROOT freigegeben wird, und die Childs noch nicht....	*/
		if (node->child != NULL) {
			sprintf(text, "!! fs_remove_vnode: Problem! Want to release node with child! NodeId:%Ld, ChildId:%Ld\n", node->id, node->child);
			debug_print(text);
		}
		if (node->parID != 0) {
			if (node->prev->child == node) node->prev->child = node->next;	/* the node was a child	*/
			else node->prev->next = node->next;

			if (node->next != NULL) node->next->prev = node->prev;		/* der Letzte der Kette!	*/
			
		}
		if ((c = node->filename) != NULL) {
			node->filename = NULL;
			free(c);
		}
		if ((at = node->attr) != NULL) {
			node->attr = NULL;
			free_all_attr(at);
		}
		free(node);
	}
	return result;
}

static int		
fs_secure_vnode(void *_ns, void *_node)
{
	char text[128];
	int			result = B_NO_ERROR;
	vnode* 		node = (vnode *)_node;	
	nspace* 	ns = (nspace*)_ns;

	sprintf(text, "## fs_secure_vnode vnode_id:%Ld\n", node->id);
	debug_print(text);
	
	return result;
}

/*	fs_open - Create a vnode cookie, if necessary, to use when
 				reading/writing a file	*/
static int		
fs_open(void *_ns, void *_node, int omode, void **cookie)
{
	dircookie* dirCookie;
	nspace* ns = (nspace*)_ns;
	vnode* node = (vnode *)_node;
	ssize_t	result = ENOMEM;
	char text[128];
	
	sprintf(text, "## fs_open: vnod_id:%Ld, omode:%x %s\n", node->id, omode, node->filename);
	debug_print(text);
	
	dirCookie = (dircookie *)malloc(sizeof(dircookie));
	if (dirCookie != NULL) {
		result = dirCookie->fd = open_node(ns, node, omode, NULL);
		dirCookie->buff = NULL;
		dirCookie->pos = dirCookie->read = 0;
			
		*cookie = (void*)dirCookie;
		if (result >= B_NO_ERROR) {
			result = B_NO_ERROR;
			sprintf(text, "-- fs_open: fd:%d result:%d\n", dirCookie->fd, result);
		} else {
			free(dirCookie);
			*cookie = NULL;
			sprintf(text, "-- fs_open: result:%d\n", result);
		}
	} else {
		sprintf(text, "-- fs_open result: %d\n", result);
	}

	debug_print(text);
	
	return result;
}


/* fs_access - checks permissions for access.	*/
static int		
fs_access(void *ns, void *_node, int mode)
{
	char text[128];
	vnode* node = (vnode *)_node;
	
	sprintf(text, "## fs_access mode:%d\n", mode);
	debug_print(text);
	
	/* we have to check, if it is allowed to access the file with the mode wanted!	*/
	/* at the moment we are always returning OK	*/
	
	return B_NO_ERROR;
}

/*===========================================================================
/	rename the filenames on the PSION and make the vnodes correct
/============================================================================*/

static int
fs_rename(void *_ns, void *_olddir, const char *oldname, void *_newdir, const char *newname)
{
	char old_file[128];
	char new_file[128];
	char text[128];
	int result = B_FILE_ERROR;
	vnode		*moveNode = NULL;
	vnode_id	vnid;
	vnode		*n;
	
	vnode* olddir = (vnode *)_olddir;
	vnode* newdir = (vnode *)_newdir;
	nspace* 	ns = (nspace*)_ns;

	find_path(ns, olddir, old_file, sizeof(old_file));	/* don't overwrite the buffer!	*/
	find_path(ns, newdir, new_file, sizeof(new_file));	/* don't overwrite the buffer!	*/

	strcat(old_file, oldname);
	strcat(new_file, newname);

	sprintf(text, "## fs_rename old:%s new:%s\n", old_file, new_file);
	debug_print(text);

	moveNode = search_vnode(ns, &vnid, olddir->id, oldname, NULL);	/* setup the node	*/
	if (moveNode == NULL) {
		result = B_ENTRY_NOT_FOUND;
	} else {
		acquire_sem(ns->dev_sem);		/* noone should change the pointers from now! */
		result = psion_rename(ns, old_file, new_file);
		if (result == B_OK) {
			strcpy(moveNode->filename, newname);	/* update the vnode-structure with the name	*/
			
			if (olddir != newdir) {
				n = moveNode->prev;										/* remove the node from the chain	*/
				if (n->next == moveNode) n->next = moveNode->next;		/* the prev is in the same level	*/
				if (n->child == moveNode) n->child = moveNode->next;	/* the first entry in a directory	*/
				if (moveNode->next != NULL) moveNode->next->prev = moveNode->prev;	/* poninter of the next	*/
			
				n = newdir;												/* plug in into the new folder	*/
				moveNode->parID = n->id;								/* lets point to our new parent	*/
				moveNode->next = n->child;								/* will be the first node in the folder	*/
				moveNode->prev = n;										/* point up to the folder	*/
				if (moveNode->next != NULL) moveNode->next->prev = moveNode;	/* point back to the new node	*/
	
				n->child = moveNode;										/* setup the parentnode				*/
			}
			
			notify_listener(B_ENTRY_MOVED, ns->id, olddir->id, newdir->id, moveNode->id, newname);	/* Lets tell the system, that there is a new folder!	*/
		}
		release_sem(ns->dev_sem);		/* let the other threads continue	*/
	}
	
	sprintf(text, "-- fs_rename result%d\n", result);
	debug_print(text);

	return result;
}

static int
fs_readlink(void *_ns, void *_node, char *buf, size_t *bufsize)
{
	debug_print("?? fs_readlink\n");
	return EINVAL;
}


/* Ab hier sinds spezielle Funktionen, die für das Protokoll benötigt werden!	*/

void WordToBytes(unsigned int i, unsigned char *p) {
    *p++ = i;
    *p = (i >> 8);
}

void LongToBytes(unsigned long l, unsigned char *p) {
    *p++ = l;
    *p++ = (l >> 8);
    *p++ = (l >> 16);
    *p = (l >> 24);
}

int16 BytesToWord(const unsigned char *c) {
	return ((int )*c | ((int )c[1] << 8)); 
}

unsigned int BytesToUword(const unsigned char *c) {
	return ((unsigned int )*c | ((unsigned int )c[1] << 8)); 
}

unsigned long BytesToUlong(const unsigned char *c) {
	return ((unsigned int )*c | ((unsigned int )c[1] << 8) | ((unsigned int )c[2] << 16) | ((unsigned int )c[3] << 24)); 
}

/* Die folgende Prozedur setzt einen Speicherbereich "vnode" mit den benötigten Daten
	die zugehörige vnode_id sind die unteren 64-Bit (daher eh die komplette Adresse)
	von dem Speicherbereich in dem der vnode abgelegt ist!

	parentID ist die vnode-ID des Verzeichnises, in dem die Datei steht!
	myID entspricht dem zurückgegebenen vnode

	Um diese allerdings wiederzufinden werden zwei Ketten aufgebaut, die einmal die nodes
	in der gleichen Ebene verbinden und einmal die Nodes darunter.
	ist der VNODE bereits geladen, dann wird kein Speicher reserviert!

	ist fileinfo = NULL, wird die Info über das Kommando RF_FINFO angefordert, ansonsten von
	fileinfo übernommen. fileinfo hat das gleiche Format, wie RF_DIRREAD oder RF_FINFO	*/

vnode *search_vnode(nspace *ns, vnode_id *myID, vnode_id parentID, const char *file, unsigned char *fileinfo)

{
	char path[128], *c;
	char text[128];
	unsigned char buffer[200], *inc_buf;
	vnode *node, *n = NULL;
	size_t len;
	status_t	port_error;

	if (strcmp(file, ".") == 0) n = ((vnode *)parentID);
	else if (strcmp(file, "..") == 0) {
		n = (vnode *)((vnode *)parentID)->parID;
		if (n == 0) *myID = ns->id;
		else *myID = n->id;					/* new	*/
		return n;
	} else if (parentID != 0) {
		n = ((vnode *)parentID)->child;		/* schau mer mal, ob wir den VNODE schon geladen haben...	*/
		while (n != NULL) {
			if (strcasecmp(file, n->filename) == 0) {	/* no matter if the filenames are upper or lowercase */
				if (fileinfo != NULL) setup_vnode(n, fileinfo);
				if (n->deleted) return NULL;
				break;					/* Schleife beenden, wir haben den VNODE gefunden!	*/
			} else n = n->next;
		}
	}
	if (n != NULL) {
		*myID = n->id;
		return n;
	}
										/* Nichts gefunden, also reservieren und belegen!	*/
	node = (vnode *)malloc (sizeof(vnode));
	if (node != NULL) {
		*myID = (vnode_id )node;
		node->id = *myID;
		node->parID = parentID;
		node->filename = strdup(file);
		node->read_all = false;			/* Initialisieren	*/
		node->read_vnode = false;
		node->stat_changed = false;		/* stat hasn't changed	*/
		node->deleted = false;
		node->attr = NULL;				/* No Attribute connected!	*/

		find_path(ns, node, path, sizeof(path));
		c = &path[strlen(path)-1];
		if (*c == '\\') *c = '\0';
		
		node->next = node->child = node->prev = NULL;
		if (parentID == 0) {		/* Das ist das Rootverzeichnis!	*/
			node->mode = S_IRWXU | S_IRWXG | S_IRWXO | S_IFDIR;
			node->size = 0;
			node->modtime = 0;
		} else {
			if (fileinfo == NULL) {
				WordToBytes(RF_FINFO, buffer);		/* Der Befehl RF_FINFO	*/
				strcpy((char *)&buffer[4], path);	/* Filenamen zu dem die Info gewünscht wird.	*/
				WordToBytes(strlen(path)+1, &buffer[2]);	/* Länge des Strings plus dem letzten 0-Byte	*/

				inc_buf = send_command(ns, buffer, &len, &port_error);
				if (inc_buf != NULL && len >= 18 && BytesToWord(&inc_buf[4]) == 0) {
					setup_vnode(node, &inc_buf[6]);
					free(inc_buf);
				} else {			/* Datei nicht gefunden!	*/
					if (node->filename != NULL) free(node->filename);
					free(node);
					if (inc_buf != NULL) free(inc_buf);
					return NULL;
				}
			} else {
				setup_vnode(node, fileinfo);
			}
			set_mime_type(ns, node);

			/* Knoten in den übergeordneten vnode einhängen!	*/
		
			n = (vnode *)parentID;
			node->next = n->child;	/* Knoten als ersten einhängen!	*/
			node->prev = n;
			if (node->next != NULL) node->next->prev = node;
			n->child = node;

			sprintf(text, "** VNODE: node%Ld added\n", node->id);
			debug_print(text);

		}
	}
	return node;
}

/* Setzt die Werte des vnode anhand der finfo-Struktur vom RFSV-Protokoll	*/

void setup_vnode(vnode *node, unsigned char *finfo) {
	mode_t mode;
	
	node->modtime = BytesToUlong(&finfo[8]);
	node->size = BytesToUlong(&finfo[4]);

	mode = BytesToUword(&finfo[2]);
	
	node->mode							= S_IRUSR|S_IRGRP|S_IROTH;			/* we can always read a file	*/
	if (mode & P_FADIR) node->mode		= S_IFDIR|S_IXUSR|S_IXGRP|S_IXOTH;
	else {
		node->mode 						|= S_IFREG;
		if (mode & P_FAEXEC) node->mode	|= S_IXUSR|S_IXGRP|S_IXOTH;
	}
	
	if (mode & P_FAWRITE) node->mode	|= S_IWUSR|S_IWGRP|S_IWOTH;
}

/* Durchsucht die Pfadstruktur rekursiv und füllt dan pfad-String	*/

/*============================== WICHTIG =============================*/
/* Max_len muß noch überprüft werden und dann mit Fehler zurückgemeldet werden!	*/

void find_path(nspace *ns, vnode *vn, char *pfad, size_t max_len) {
	if (vn->parID != 0) {
		find_path(ns, (vnode *)vn->parID, pfad, max_len);
		strcat(pfad, vn->filename);
		if (vn->mode & S_IFDIR) strcat(pfad, "\\");
	} else {
		strcpy(pfad, ns->device);
		strcat(pfad, vn->filename);
	}
}

/* Diese Routine öffnet einen vnode im PSION.
	Übergeben wird der VNODE und zurückgegeben, wird der Handler, oder ein Fehlercode!
/	*name is the name, when the opmode is for creating a new file. otherwise it is NULL	*/

int	open_node(nspace *ns, vnode *vn, int omode, const char *name) {
	int result, psi_omode;
	unsigned char buffer[128], *inc_buf;
	char *c;
	size_t len;
	status_t	port_error;
	
	c = (char *)&buffer[6];
	find_path(ns, vn, c, sizeof(buffer)-6);		/* Größe des Buffers!	*/
	if (name != NULL && (omode & O_CREAT)) {	/* expand the filename only, when a file have to be createn	*/
		strcat(c, name);						
	}

	psi_omode = P_FOPEN|P_FRANDOM;				/* its the standard, we want to open a file	*/
	if (omode & O_TRUNC || omode & O_CREAT) {
		vn->size = 0;				/* we start from 0-bytes	*/
		psi_omode &= ~P_FSER_MASK;	/* Delete the Fileservicebits	*/
		psi_omode |= P_FREPLACE;	/* Truncate the file	*/
	}
	if (omode & O_APPEND) {
		psi_omode &= ~P_FSER_MASK;	/* Delete the Fileservicebits	*/
		psi_omode |= P_FAPPEND;		/* Append the file		*/
	}
	if (((omode & O_RWMASK) == O_RDWR) || (omode & O_WRONLY))	{	/* we want to write to the file	*/
		psi_omode |= P_FUPDATE;
	}
	
	WordToBytes(RF_FOPEN, buffer);				/* Der Befehl P_FOPEN	*/
	if ((vn->mode & S_IFDIR) && !(omode & O_CREAT)) {
		WordToBytes(P_FDIR, &buffer[4]);		/* Wir wollen ein Verzeichnis öffnen	*/
		strcat(c, "*.*");						/* Der PSION will, daß beim Verzeichnisöffnen immer ein *.* angehängt wird!	*/
	} else {
		WordToBytes(psi_omode, &buffer[4]);		/* Wir wollen eine normale Datei öffnen */
	}
	
	WordToBytes(strlen(c)+3, &buffer[2]);		/* Länge berechnen! + 0-Byte + Open-Status	*/

	inc_buf = send_command(ns, buffer, &len, &port_error);	/* Also senden wir das Öffnen kommando an den PSION	*/
	if (inc_buf != NULL) {						/* Wir haben eine gültige Antwort erhalten	*/
		if (len >= 6) {	
			switch (BytesToWord(&inc_buf[4])) {	/* Hier steht das Ergebnis der Datei öffnen	*/
				case 0:				result = (int )BytesToWord(&inc_buf[6]);	/* Filehandle übernehmen!	*/
									break;
				case E_FILE_NAME:	result = ENAMETOOLONG; break;
				case E_FILE_NXIST:	
				case E_FILE_DEVICE:	
				case E_FILE_DIR:	result = ENOENT; break;
				default:			result = EINVAL; break;
			}
		} else result = EIO;					/* Wir haben nicht genügen Bytes erhalten, daher IO-Error	*/
		free (inc_buf);
	} else {
		if (port_error == B_BAD_PORT_ID) result = B_IO_ERROR;
		else result = ENOMEM;
	}
	
	return result;
}

/*	Diese Prozedur sendet einen Befehl and den WriteThread und wartet auf eine Antwort.
	Sie wird durch eine Semaphore geschützt, sodaß die richtige Antwort auch zum richtigen
	Aufrufer zurückkommt!

	Im buffer an Byte 2+3 muß die korrekte Länge vorhanden sein!	
	error is the returncode of the portfunction.
	*/

unsigned char *send_command(nspace *ns, unsigned char *buffer, size_t *msg_len, status_t *error) {
	size_t		buf_len;
	unsigned char *inc_buf;
	int32		msg;
	status_t	result;
	char text[128];
	
	buf_len = BytesToUword(&buffer[2]) + 4; /*Die Zwei Befehls und zwei Längenbytes dazuzählen!	*/
	if (acquire_sem(ns->port_sem) != B_NO_ERROR) {
		debug_print("PANIC: Semaphore not available!\n");
		return (NULL);	/* es ist ein Fehler aufgetreten, die Semaphore existiert nicht mehr!	*/
	}
	
	*error = write_port(ns->pid_write, PSION_CMD + ns->cid, buffer, buf_len);
	if (*error >= B_NO_ERROR) {				/* no problem to send the command	*/

#ifdef DEBUG_FILE
dbg_print("\nPC->PSION ", buf_len, buffer);
#endif

		*error = port_buffer_size_etc(ns->pid_read, B_TIMEOUT, 10000000L); /* 10 Sekunden auf Nachrichten warten!	*/
		if (*error >= B_NO_ERROR) {	
			*msg_len = (size_t )*error;
			inc_buf = (unsigned char *)malloc(*msg_len);	/* Speicher für die Nachricht reservieren.	*/
			if (inc_buf != NULL) {
				*error = read_port_etc(ns->pid_read, &msg, (void *)inc_buf, *msg_len, B_TIMEOUT, 10000000L);	/* 10 Sekunden warten	*/
				if (*error >= B_NO_ERROR) {				/* Nachricht wurde korrekt eingelesen!	*/

#ifdef DEBUG_FILE
dbg_print("\nPSION->PC ", *msg_len, inc_buf);
#endif

					release_sem(ns->port_sem);				/* Semaphore wieder freigeben!	*/
					return(inc_buf);					/* Buffer übergeben. Das Freigeben muß der Aufrufer übernehmen!	*/
				}
			}
		}
	}
	
	if (*error == B_BAD_PORT_ID) {		/* PalmBeach-Port isn't longer available	*/
		ns->pb_running = false;			/* PalmBeach not found panic error			*/
		sprintf(text, "PANIC: PalmBeach Port not available!\n");
	} else sprintf(text, "PANIC: Porterror: %ld\n", *error);

	debug_print(text);
	
	release_sem(ns->port_sem);			/* Hierher kommen wir nur im Fehlerfall! Semaphore wieder freigeben!	*/
	if (inc_buf != NULL) free(inc_buf);	/* den reserveriten Speicher wieder freigeben	*/
	return(NULL);						/* Und Fehlercode zurückschicken!	*/
}

/* This function is used to send the Debuginformation to a file */

void dbg_print(char *head, size_t len, unsigned char *buffer)
{
	int fd_debug;
	size_t i;
	char dbg[10];
	
	if (*debugFile != '\0') {
		if ((fd_debug = open(debugFile, O_APPEND|O_RDWR|O_CREAT)) > 0) {
			write(fd_debug, head, strlen(head));
			for (i = 0; i < len; i++) {
				sprintf(dbg, "%02x ", buffer[i]);
				write(fd_debug, dbg, 3);
			}
			close(fd_debug);
		}
	}
}

