/*============================================================================
/ psionfs_dir.c
/ © Michael Pieper 03. Oct. 2000
/      Last change 01. Nov. 2000 Michael Pieper
/============================================================================
/
/	This c-File includes all "Directory"-Functions for the PSION-FileSystem
/
/============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <TypeConstants.h>

/*===========================================================================
/	We also need our own Headerfiles
/============================================================================*/

#include "psionfs.h"
#include "psionfs_dir.h"	/* Including all the Directory-Filesystem-Functions	*/
#include "psionfs_rfsv.h"	/* Including all PSION-Commands						*/

#include "mc_rfile.h"
#include "p_file.h"

#include "debug.h"

/*===========================================================================
/	creates fs-specific "cookie" struct that keeps track of where
/ 	you are at in reading through directory entries in fs_readdir.
/============================================================================*/

int	psionfs_opendir(void *_ns, void *_node, void **cookie)
{
	dircookie* dirCookie;
	unsigned char *gen;
	nspace* ns = (nspace*)_ns;
	vnode* node = (vnode *)_node;
	ssize_t	result = ENOMEM;
	char text[128];
	
	sprintf(text, "## fs_opendir: vnod_id:%Ld, parid:%Ld %s\n", node->id, node->parID, node->filename);
	debug_print(text);
	
	dirCookie = (dircookie *)malloc(sizeof(dircookie));
	if (dirCookie != NULL) {
		result = dirCookie->fd = open_node(ns, node, 0, NULL);
		dirCookie->pos = 0;
		gen = (unsigned char * )malloc(37);	/* Speicher für "." und ".." reserviert!*/
		memset(gen, 0, 37);
		gen[16] = '.';
		gen[18+16] = '.';
		gen[18+17] = '.';
		dirCookie->buff = gen;
		dirCookie->read = 37;				/* "." und ".." generiert */
		*cookie = (void*)dirCookie;
		if (result >= B_NO_ERROR) {
			result = B_NO_ERROR;
			sprintf(text, "-- fs_opendir: fd:%d result:%d\n", dirCookie->fd, result);
		} else {
			free(dirCookie);
			*cookie = NULL;
			sprintf(text, "-- fs_opendir: result:%d\n", result);
		}
	} else {
		sprintf(text, "-- fs_opendir result:%d\n", result);
	}

	debug_print(text);
	
	return result;
}

/*===========================================================================
/	fs_readdir - read 1 or more dirents, keep state in cookie, return
/				0 when no more entries.
/===========================================================================*/

int	psionfs_readdir(void *_ns, void *_node, void *_cookie, long *num, 
					struct dirent *buf, size_t bufsize)
{
	char text[128];
	unsigned char buffer[10];
	size_t 		len;
	int 		result = ENOENT;
	vnode 		*node = (vnode *)_node, *n;
	dircookie 	*dirc = (dircookie *)_cookie;
	nspace		*ns = (nspace*)_ns;
	vnode_id	dummy;
	status_t	port_error;
	
	sprintf(text, "## fs_readdir: vnod_id:%Ld, buffsize:%ld\n", node->id, bufsize);
	debug_print(text);

	if (dirc->pos >= dirc->read) {			// alle gelesenen schon ausgegeben!
		if (dirc->buff != NULL) free(dirc->buff); // dann vergessen wir den alten Buffer!
		WordToBytes(RF_FDIRREAD, buffer);	// Der Befehl RF_FDIRREAD
		WordToBytes(2, &buffer[2]);			// Länge immer 2
		WordToBytes(dirc->fd, &buffer[4]);	// Den Filedeskriptor schliessen

		dirc->buff = send_command(ns, buffer, &len, &port_error);
		if (dirc->buff != NULL && len >= 8) {
			if (BytesToWord(&dirc->buff[4]) == 0) {
				dirc->read = len;
				dirc->pos = 8;				// hier beginnt der erste Datensatz!
				result = B_NO_ERROR;
			}
		} else result = ENOMEM;
	}
	if (port_error == B_BAD_PORT_ID) {	/* the Port isn't available any longer!	PalmBeach crashed */
		dirc->pos = dirc->read;			/* we should fake, that we got all dir-entries	*/
	}
	if (dirc->pos >= dirc->read) {		// Keine weiteren Dateien vorhanden!
		node->read_all = true;			// Verzeichnis wurde komplett eingelesen!
		result = B_NO_ERROR;			// Kein Fehler aufgetreten
		*num = 0;						// Aber kein File mehr eingelesen
	} else {
		result = B_NO_ERROR;
		strcpy(buf->d_name, (char *)&dirc->buff[dirc->pos+16]);
		buf->d_reclen = strlen(buf->d_name);
		n = search_vnode(ns, &dummy, node->id, buf->d_name, &dirc->buff[dirc->pos]);
		buf->d_dev = buf->d_pdev = ns->id;
		buf->d_ino = dummy;	
		buf->d_pino = node->id;

		dirc->pos += (16 + 1 + buf->d_reclen);		// 0-Byte mitzählen
		*num = 1;
		sprintf(text, "-- fs_readdir: File >%s< d_ino:%Ld d_pino:%Ld\n", buf->d_name, buf->d_ino, buf->d_pino);
		debug_print(text);
	}
	sprintf(text, "-- fs_readdir result:%d\n", result);
	debug_print(text);
	
	return result;
}

/*===========================================================================
/	fs_rewinddir - set cookie to represent beginning of directory, so
/					later fs_readdir calls start at beginning.
/===========================================================================*/

int	psionfs_rewinddir(void *ns, void *node, void* cookie)
{
	debug_print("## fs_rewinddir\n");
	psionfs_closedir(ns, node, cookie);	/* there is no rewind in the PSION Filesystem available */
	psionfs_opendir(ns, node, &cookie);	/* so we have to close and open again */
	return B_NO_ERROR;
}

/*===========================================================================
/	fs_closedir - Do whatever you need to to close a directory (sometimes
/				nothing), but DON'T free the cookie!
/===========================================================================*/

int	psionfs_closedir(void * ns, void *node, void *_cookie)
{
	char text[128];
	
	dircookie *dirc = (dircookie*)_cookie;
	
	sprintf(text, "## fs_closedir fd:%d\n", dirc->fd);
	debug_print(text);	/* We don't have to do anything. The PSION-Channel is closed in free-Dir-Cookie*/
	
	return B_NO_ERROR;
}

/*===========================================================================
/	fs_free_dircookie - Free the fs-specific cookie struct
/					This cookie was generated in open_dir
/===========================================================================*/

int	psionfs_free_dircookie	(void *_ns, void *node, void *_cookie)
{
	char text[128];
	dircookie *dirc = (dircookie*)_cookie;
	nspace* ns = (nspace*)_ns;	

	sprintf(text, "## fs_free_dircookie fd:%d\n", dirc->fd);
	debug_print(text);

	psion_close(ns, dirc);
	
	return B_NO_ERROR;
}

/*===========================================================================
/	fs_mkdir - generates a new directory
/				dir is the vnode for the directory in which the dir should be generated
/===========================================================================*/

int	psionfs_mkdir			(void *_ns, void *_node, const char *name,	int perms)
{
	int result;
	char buffer[128];

	char text[128];
	nspace* ns = (nspace*)_ns;	
	vnode *node = (vnode*)_node;

	sprintf(text, "## fs_mkdir: name:>%s<, vnod_id:%Ld, perms %x\n", name, node->id, perms);
	debug_print(text);

	find_path(ns, node, buffer, sizeof(buffer));	/* don't overwrite the buffer!	*/
    if (strlen(buffer) + strlen(name) + 1 > sizeof(buffer)) return ENOMEM;	/* not enough memory	*/
	strcat(buffer, name);		/* Add the requested Directoryname			*/
	
	sprintf(text, "-- fs_mkdir: Fullpathname:>%s<\n", buffer);
	debug_print(text);

	result = psion_mkdir (ns, buffer);

	notify_listener(B_ENTRY_CREATED, ns->id, node->id, 0, 0, name);	/* Lets tell the system, that there is a new folder!	*/

	sprintf(text, "-- fs_mkdir beendet mit %x\n", result);
	debug_print(text);
	
	return result;	
}

/*===========================================================================
/	fs_rmdir - has to delete a directory
/===========================================================================*/

int	psionfs_rmdir			(void *_ns, void *_node, const char *name)
{
	char text[128];
	int			result = B_NO_ERROR;
	nspace* 	ns = (nspace*)_ns;	
	vnode*		dir = (vnode*)_node;
	vnode*		unlinkNode = NULL;
	vnode_id	vnid;
	dircookie*	cookie;
	
	sprintf(text, "## fs_rmdir: name:>%s<, vnod_id:%Ld\n", name, dir->id);
	debug_print(text);

	unlinkNode = search_vnode(ns, &vnid, dir->id, name, NULL);	/* find the node	*/

	if (unlinkNode == NULL) return B_ENTRY_NOT_FOUND;			/* file not found!			*/
	if (get_vnode(ns->id, vnid, (void**)&unlinkNode) != 0) return B_BAD_VALUE;	/* initialise it on the kernel	*/
 
	if (unlinkNode->mode & S_IFREG) return B_NOT_A_DIRECTORY;	/* want to rmdir a file	*/
	
	remove_vnode(ns->id, unlinkNode->id);						/* remove the vnod-id			*/
	unlinkNode->deleted = true;									/* set the mark, that the node is deleted	*/
	notify_listener(B_ENTRY_REMOVED, ns->id, dir->id, 0, unlinkNode->id, NULL);	/* Lets tell the system, that there is a file deleted!	*/

	put_vnode(ns->id, unlinkNode->id);							/* we aren't interested in the data any longer	*/
	return result;
}

