/*============================================================================
/ psionfs_rfsv.c
/ © Michael Pieper 21. Oct. 1999
/     Last change: 11. Nov. 2000 Michael Pieper
/============================================================================
/
/	This c-File includes all Commands, which can be sent to the PSION
/	At the moment it is only using the EPOC 16-Commands
/
/============================================================================*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/*===========================================================================
/	We also need our own Headerfiles
/============================================================================*/

#include "psionfs_rfsv.h"

#include "p_file.h"
#include "mc_rfile.h"

#include "debug.h"

/*===========================================================================
/	closes an filedescriptor and deletes the cookie
/	PSION-FunctionId: (Psion 3c: $02)
/============================================================================*/

int psion_close (nspace *ns, dircookie *cookie) {

	unsigned char 	buffer[6], *inc_buf;	/* wee need exactly 6 Bytes Sendbuffer	*/
	ssize_t			result = B_FILE_ERROR;
	size_t			len;
	status_t		port_error;

	WordToBytes(RF_FCLOSE, buffer);			/* The Command RF_FLCLOSE		*/
	WordToBytes(2, &buffer[2]);				/* is always 2 Bytes long		*/
	WordToBytes(cookie->fd, &buffer[4]);	/* close the filedescriptor		*/

	inc_buf = send_command(ns, buffer, &len, &port_error);
	if (inc_buf != NULL) {
		if (len >= 4) {
			switch (BytesToWord(&inc_buf[4])) {
				case 0: 
					result = B_OK; 
					if (cookie != NULL) {
						if (cookie->buff != NULL) free(cookie->buff);
						free(cookie);
					}
					break;
			}
		} else result = B_IO_ERROR;			/* we got not enough bytes, so it is a IO-Error	*/
		free(inc_buf);
	} else result = B_NO_MEMORY;			/* not enogh memory available	*/

	if (port_error == B_BAD_PORT_ID) {		/* PalmBeach crashed. report the correct close	*/
		result = B_OK;
		if (cookie != NULL) {
			if (cookie->buff != NULL) free(cookie->buff);
			free(cookie);
		}
	}

	return result;
}

/*===========================================================================
/	lets read a file
/	the return value is the ammount of bytes
/	PSION-FunctionId: (Psion 3c: $04)
/============================================================================*/

ssize_t psion_read (nspace *ns, dircookie *cookie, char *buf, size_t length) {

	unsigned char 	buffer[8], *inc_buf;	/* wee need exactly 8 Bytes Sendbuffer	*/
	size_t			read_len, actual_read, sum_read = 0;
	ssize_t			result = B_FILE_ERROR;
	status_t		port_error;
	int				l;
	char text[128];
	
	WordToBytes(RF_FREAD, buffer);			/* The Command RF_FREAD		*/
	WordToBytes(4, &buffer[2]);				/* is always 4 Bytes long	*/
	WordToBytes(cookie->fd, &buffer[4]);	/* Set the filedescriptor	*/
	
	do {
		l = (length-sum_read > MAX_BLOCK) ? MAX_BLOCK : (unsigned int ) length-sum_read;
	
		WordToBytes(l, &buffer[6]);		/* How much Bytes are we allowed to read?	*/

		inc_buf = send_command(ns, buffer, &read_len, &port_error);
		if (inc_buf != NULL) {
			if (read_len >= 4) {
				
				switch (BytesToWord(&inc_buf[4])) {
					case 0:
						result = B_NO_ERROR;
						actual_read = BytesToUword(&inc_buf[2])-2;
						memcpy(&buf[sum_read], &inc_buf[6], actual_read);

						sum_read 		+= actual_read;
						cookie->pos		+= actual_read;
						break;
					case E_FILE_EOF:	length = 0; result = B_NO_ERROR; break;	/* set length to 0 that we can leave the loop	*/
					default:			result = B_FILE_ERROR; break;
				}
			} else result = B_IO_ERROR;							/* we got not enough bytes, so it is a IO-Error	*/
			free(inc_buf);
		} else {
			if (port_error == B_BAD_PORT_ID) result = B_IO_ERROR;	/* PalmBeach crashed, report IO-ERROR	*/
			else result = B_NO_MEMORY;							/* not enogh memory available	*/
		}
	} while ((length > sum_read) && (result == B_NO_ERROR));	/* repeat until no error appeared	*/

	if (sum_read > 0) result = sum_read;						/* if we got some bytes return with no error	*/

	return result;
}

/*===========================================================================
/	walks through a directory. returns for every Entry a structure
/	it returns not all and not only one entry!
/	PSION-FunctionId: (Psion 3c: $06)
/============================================================================*/

/*int psion_dirread(nspace *ns, int fd, int *count, struct dirent *buf, size_t buffsize) {
}*/

/*===========================================================================
/	lets write a file
/	the return value is the ammount of bytes
/	PSION-FunctionId: (Psion 3c: $0A)
/============================================================================*/

ssize_t psion_write (nspace *ns, dircookie *cookie, const char *buf, size_t length) {

	unsigned char	buffer[MAX_BLOCK], *inc_buf;
	size_t			read_len, sum_write = 0;
	int16			erg; 
	int				result = B_NO_ERROR;
	size_t			len;
	status_t		port_error;
	
	WordToBytes(RF_FWRITE, buffer);			/* The Commandcode of RF_FWRITE	*/
	WordToBytes(cookie->fd, &buffer[4]);	/* set the filedexcriptor		*/

	do {
		len = (length-sum_write > MAX_BLOCK-8) ? MAX_BLOCK-8 : (unsigned int ) length-sum_write;

		WordToBytes(2 + len, &buffer[2]);	/* set the length */
	
		memcpy(&buffer[6], &buf[sum_write], len);		
	
		inc_buf = send_command(ns, buffer, &read_len, &port_error);
		if (inc_buf != NULL) {
			if (read_len >= 4) {
				erg = BytesToWord(&inc_buf[4]);	
				switch (erg) {
					case 0:
						sum_write		+= len;
						cookie->pos		+= len;
						break;
					case E_FILE_RDONLY:	result = B_READ_ONLY_DEVICE;	break;
					default:			result = B_BAD_VALUE;			break;
				}
			} else {
				result = B_IO_ERROR;			/* we got not enough bytes, so it is a IO-Error	*/
				free (inc_buf);
				break;
			}
			free (inc_buf);
		} else {
			if (port_error == B_BAD_PORT_ID) result = B_IO_ERROR;
			else result = B_NO_MEMORY;			/* not enogh memory available	*/
		}
	} while (length > sum_write && result == B_NO_ERROR);
	
	if (sum_write > 0) result = sum_write;
	
	return result;
}

/*===========================================================================
/	Do a fseek on a defined cookie (file)
/	PSION-FunctionId: (Psion 3c: $0C)
/============================================================================*/

int psion_fseek (nspace *ns, dircookie *cookie, int seekmode, ssize_t offset) {

	unsigned char 	buffer[12], *inc_buf;	/* wee need exactly 12 Bytes Sendbuffer	*/
	int				result = B_FILE_ERROR;
	size_t			read_len;
	status_t		port_error;

	WordToBytes(RF_FSEEK, buffer);			/* The Commandcode of RF-FSEEK	*/
	WordToBytes(8, &buffer[2]);				/* The length is always 8		*/
	WordToBytes(cookie->fd, &buffer[4]);	/* The filedescriptor			*/
	LongToBytes(offset, &buffer[6]);		/* The fileposition				*/
	switch(seekmode) {
		case SEEK_SET:	
			WordToBytes(P_FABS, &buffer[10]);			/* absolute Mode	*/
			if (cookie->pos == offset) return(B_OK);	/* no change!	*/
			break;
		case SEEK_CUR:
			WordToBytes(P_FCUR, &buffer[10]);		/* relative to current pos	*/
			if (offset == 0) return(B_OK);			/* no move	*/
			break;
		case SEEK_END:	
			WordToBytes(P_FEND, &buffer[10]);		/* relative to the end	*/
			if (offset > 0) return(B_FILE_ERROR);	/* can't move over the end	*/
			break;
		default:
			return B_FILE_ERROR;
	}
	
	inc_buf = send_command(ns, buffer, &read_len, &port_error);
	if (inc_buf != NULL) {
		if (read_len >= 6) {
			switch (BytesToWord(&inc_buf[4])) {
				case 0:
					cookie->read = BytesToUlong(&inc_buf[6]);
					result = B_OK;
					break;
			}
		} else result = B_IO_ERROR;			/* we got not enough bytes, so it is a IO-Error	*/
		free(inc_buf);
	} else {
		if (port_error == B_BAD_PORT_ID) result = B_IO_ERROR;	/* PalmBeach crashed	*/
		else result = B_NO_MEMORY;			/* not enogh memory available	*/
	}
	
	return result;
}

/*===========================================================================
/	walks through a directory. returns for every Entry a structure
/	it returns not all and not only one entry!
/	PSION-FunctionId: (Psion 3c: $06)
/============================================================================*/

int psion_rename(nspace *ns, const char *old_file, const char *new_file) {

	unsigned char 	buffer[MAX_BLOCK], *inc_buf;	/* wee need a maximum Sendbuffer	*/
	int				result = B_FILE_ERROR;
	size_t			read_len;
	size_t			len_old, len_new;
	status_t		port_error;
	
	len_old = strlen(old_file);
	len_new = strlen(new_file);
	WordToBytes(RF_RENAME, buffer);					/* The command RF_RENAME							*/
	WordToBytes(len_old + len_new + 2, &buffer[2]);	/* We have to calculate the Length! + 2 - 0-Byte 	*/
	strcpy((char *)&buffer[4], old_file);			/* copy the complete old filename					*/
	strcpy((char *)&buffer[5+len_old], new_file);	/* copy the complete new filename					*/

	inc_buf = send_command(ns, buffer, &read_len, &port_error);	/* Now send the Command to the PSION */
	if (inc_buf != NULL) {							
		if (read_len >= 4) {	
			switch (BytesToWord(&inc_buf[4])) {		/* This is the resultcode.	*/
				case 0:				result = B_OK;					break;
				default:			result = B_FILE_ERROR;			break;
			}
		} else result = B_IO_ERROR;			/* we got not enough bytes, so it is a IO-Error	*/
		free (inc_buf);
	} else {
		if (port_error == B_BAD_PORT_ID) result = B_IO_ERROR;	/* PalmBeach crashed	*/
		else result = B_NO_MEMORY;			/* not enogh memory available	*/
	}
	return result;
}

/*===========================================================================
/	Deletes a file or a folder
/	PSION-FunctionId: (Psion 3c: $14)
/============================================================================*/

int psion_delete (nspace *ns, const char *file) {
	unsigned char 	buffer[MAX_BLOCK], *inc_buf;	/* wee need a maximum Sendbuffer	*/
	int				result = B_FILE_ERROR;
	size_t			read_len;
	status_t		port_error;
	
	WordToBytes(RF_DELETE, buffer);					/* The command RF_DELETE						*/
	WordToBytes(strlen(file)+1, &buffer[2]);		/* We have to calculate the Length! + 0-Byte 	*/
	strcpy((char *)&buffer[4], file);				/* copy the complete foldername					*/

	inc_buf = send_command(ns, buffer, &read_len, &port_error);	/* Now send the Command to the PSION */
	if (inc_buf != NULL) {							
		if (read_len >= 4) {	
			switch (BytesToWord(&inc_buf[4])) {		/* This is the resultcode.	*/
				/* it must be always ok, because this function isn't corresponding to a specified command	*/
				case 0:				result = B_OK;					break;
				default:			result = B_NO_ERROR;			break;
			}
		} else result = B_IO_ERROR;			/* we got not enough bytes, so it is a IO-Error	*/
		free (inc_buf);
	} else {
		if (port_error == B_BAD_PORT_ID) result = B_IO_ERROR;	/* PalmBeach crashed	*/
		else result = B_NO_MEMORY;			/* not enogh memory available	*/
	}
	return result;
}

/*===========================================================================
/	Sets the Filestatus of one file
/	PSION-FunctionId: (Psion 3c: $18)
/============================================================================*/

int psion_sfstat (nspace *ns, const char *file, uint16 status, uint16 mask) {
	return B_NO_ERROR;
}

/*===========================================================================
/	creates a folder on the PSION
/	PSION-FunctionId: (Psion 3c: $1C)
/============================================================================*/

int psion_mkdir (nspace *ns, const char *folder) {

	unsigned char 	buffer[MAX_BLOCK], *inc_buf;	/* wee need a maximum Sendbuffer	*/
	int				result = B_FILE_ERROR;
	size_t			read_len;
	status_t		port_error;
	
	WordToBytes(RF_MKDIR, buffer);					/* The command RF_MKDIR							*/
	WordToBytes(strlen(folder)+1, &buffer[2]);		/* We have to calculate the Length! + 0-Byte 	*/
	strcpy((char *)&buffer[4], folder);				/* copy the complete foldername					*/
	
	inc_buf = send_command(ns, buffer, &read_len, &port_error);	/* Now send the Command to the PSION */
	if (inc_buf != NULL) {							
		if (read_len >= 4) {	
			switch (BytesToWord(&inc_buf[4])) {	/* This is the resultcode	*/
				case 0:				result = B_OK;					break;
				case E_FILE_NAME:	result = B_BAD_VALUE;			break;
				case E_FILE_PROTECT:result = B_READ_ONLY_DEVICE;	break;
				case E_FILE_EXIST:	result = B_FILE_EXISTS;			break;
				case E_FILE_DEVICE:	
				case E_FILE_DIR:	result = B_ENTRY_NOT_FOUND; 	break;

			}
		} else result = B_IO_ERROR;			/* we got not enough bytes, so it is a IO-Error	*/
		free (inc_buf);
	} else {
		if (port_error == B_BAD_PORT_ID) result = B_IO_ERROR;	/* PalmBeach crashed	*/
		else result = B_NO_MEMORY;			/* not enogh memory available	*/
	}
	return result;
}

/*===========================================================================
/	Sets the Filedate of one file
/	PSION-FunctionId: (Psion 3c: $28)
/============================================================================*/

int psion_sfdate (nspace *ns, const char *file, uint32 seconds) {

	unsigned char 	buffer[MAX_BLOCK], *inc_buf;	/* wee need a maximum Sendbuffer	*/
	int				result = B_FILE_ERROR;
	size_t			read_len;
	status_t		port_error;
	
	WordToBytes(RF_SFDATE, buffer);					/* The command RF_SFDATE						*/
	WordToBytes(strlen(file)+1+4, &buffer[2]);		/* We have to calculate the Length! + 0-Byte + the dateinformation 	*/
	LongToBytes(seconds, &buffer[4]);				/* The Seconds since 1.1.1970					*/
	strcpy((char *)&buffer[8], file);						/* The filename									*/

	inc_buf = send_command(ns, buffer, &read_len, &port_error);	/* Now send the Command to the PSION */
	if (inc_buf != NULL) {							
		if (read_len >= 4) {	
			switch (BytesToWord(&inc_buf[4])) {	/* This is the resultcode	*/
				case 0:				result = B_OK;					break;
				case E_FILE_NAME:	result = B_BAD_VALUE;			break;
				case E_FILE_PROTECT:result = B_READ_ONLY_DEVICE;	break;

			}
		} else result = B_IO_ERROR;			/* we got not enough bytes, so it is a IO-Error	*/
		free (inc_buf);
	} else {
		if (port_error == B_BAD_PORT_ID) result = B_IO_ERROR;	/* PalmBeach crashed	*/
		else result = B_NO_MEMORY;			/* not enogh memory available	*/
	}
	return result;
}
